@@ -10,7 +10,7 @@
 
 int buttonDown = 0;
 
-W_Window baseWin;
+W_Window shellWin, baseWin;
 
 W_Image *playerShip, *playerTorp, *enemyTorp, *shieldImage;
 
@@ -30,6 +30,11 @@ int getting_name = 0;
 int counter;
 
 int mouseControl = 0;
+int wantStars = 1;
+
+/* Modified slightly to better fit on 800x600 in fullscreen mode */
+int winwidth  = 468; /* was 400 */
+int winheight = 596; /* was 511 */
 
 int alien_shape[ALIENSHAPES] = {
     I_ALIEN1,
@@ -55,6 +60,7 @@ int weapon = 0;
 int maxtorps = 4, numtorps = 0;
 
 int plx = 200;
+int pldead = 0;
 
 int movespeed = MINSPEED;
 
@@ -67,6 +73,8 @@ int playSounds=1;
 #endif
 
 int plshield = 0;
+int shieldsleft = 0;
+int shieldon = 0;
 
 int title_page=0, pagetimer=300;
 
