@@ -1,11 +1,12 @@
 #include "Wlib.h"
+#include "struct.h"
 
-extern W_Window gal, baseWin;
+extern W_Window gal, shellWin, baseWin;
 
 extern char *imagedir;
 extern int verbose_image_loading;
 extern int forceMono;
-extern int xpm, useOR;
+extern int xpm, useOR, useClipMask, useBuffered;
 
 extern W_Color backColor;
 
@@ -18,6 +19,8 @@ extern int score;
 extern int paused;
 
 extern int ships;
+extern int shieldsleft;
+extern int shieldon;
 
 extern int level, metaLevel;
 extern int startLevel;
@@ -30,6 +33,10 @@ extern int getting_name;
 extern int counter;
 
 extern int mouseControl;
+extern int wantStars;
+
+extern int winwidth;
+extern int winheight;
 
 extern int alien_shape[];
 
@@ -37,6 +44,7 @@ extern int weapon;
 extern int maxtorps, numtorps;
 
 extern int plx;
+extern int pldead;
 
 extern int movespeed;
 
@@ -55,3 +63,6 @@ extern int title_page, pagetimer;
 extern int gotlemon;
 
 extern struct alien aliens[];
+
+extern struct js_state js_curstate;
+extern int js_device;
