@@ -55,6 +55,8 @@ struct window {
     W_Callback handle_button;
     W_Callback handle_expose;
 #endif				/* 0 */
+    W_Window wm_window;
+    Window fs_window;
 };
 
 struct stringList {
