@@ -7,6 +7,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
@@ -14,10 +15,10 @@
 # include <unistd.h>
 #endif
 #include "libsprite/Wlib.h"
+#include "struct.h"
 #include "data.h"
 #include "images.h"
 #include "defs.h"
-#include "struct.h"
 #include "paths.h"
 #include "proto.h"
 #include "sound.h"
@@ -31,55 +32,71 @@ static int livecount = 0;
 static int starspeed = 1;
 static int attacking = 0, maxattacking, entering=0;
 static int maxetorps = 5, numetorps=0;
-static int pldead = 0;
 static int plflash = 50;
+static int fullscreen = 0;
+#ifndef ORIGINAL_XGALAGA
+static int shots = 0;
+static int hits = 0;
+#endif
 
 #define convoy_x_pos(i) (convoyx+(20 * (i - 10 * (i/10))))
 #define convoy_y_pos(i) (20 + (20*(i/10)))
 
 static int moves[16][2] = {
-    0,-4,
-    1,-4,
-    3,-3,
-    4,-1,
-    4,0,
-    4,1,
-    3,3,
-    1,4,
-    0,4,
-    -1,4,
-    -3,3,
-    -4,1,
-    -4,0,
-    -4,-1,
-    -3,-3,
-    -1,-4
+    {0,-4},
+    {1,-4},
+    {3,-3},
+    {4,-1},
+    {4,0},
+    {4,1},
+    {3,3},
+    {1,4},
+    {0,4},
+    {-1,4},
+    {-3,3},
+    {-4,1},
+    {-4,0},
+    {-4,-1},
+    {-3,-3},
+    {-1,-4}
 };
 
 void
 xgal_exit(int v)
 {
     /*    W_AutoRepeatOn();*/
+    /* Destroy our main window so the fullscreen mode gets unset if we're
+       running fullscreen. (We should really clean up much more here!) */
+    W_DestroyWindow(shellWin);
 #ifdef SOUND
     kill_sound();
 #endif
     /*    sleep(1);*/ /* Without this, the auto-repeat request fails on my machine... */
+    /* Note if we ever need this autorepeat thingie again, the sleep can and should be replaced by a call to XSync() */
     exit(v);
 }
 
 
 static void print_usage()
 {
+#ifdef SOUND 
+#define PRINT_SOUND	"-nosound             Turn sound OFF\n"
+#else
+#define PRINT_SOUND	""
+#endif
+
     printf("\
 XGalaga v%s\n\
 Copyright (c) 1995-1998 Joe Rumsey\n\
-<mrogre@mediaone.net>\n\
+Contributions by Hermann Riedel\n\
 Command line options:\n\
 -scores              Prints out the high score files and exits\n\
 -display <display>   Set your display\n\
 -mouse               Use mouse control (same as 'm' at the title screen)\n\
 -keyboard            Use keyboard control (same as 'k')\n%s\
 -level <number>      Choose starting level (>= 1)\n\
+-window              Start in windowed mode instead of fullscreen\n\
+-winsize <WxH>       Window size (default 468 x 596)\n\
 -b                   turn buffered mode off, use this if it runs\n\
 too slowly.  Will cause flicker, maybe lots,\n\
 maybe only a little.\n\
@@ -96,12 +113,8 @@ Keyboard commands:\n\
 p - pauses\n\
 q - end this game\n\
 b - Toggle buffering (flicker vs. speed.)\n\
-o - Toggle OR mode.  Don't use this, it sucks.\n", VERSION,
-#ifdef SOUND
-           "  -nosound             Turn sound OFF\n"
-#else
-           ""
-#endif
+alt + enter - Toggle fullscreen - window\n", VERSION,
+	PRINT_SOUND
            );
 }
 
@@ -140,8 +153,10 @@ static void undo_stars()
 {
     int i;
     
-    for(i=0;i<NUMSTARS;i++) {
-        W_DrawPoint(baseWin, stars[i].x, stars[i].y, W_Black);
+    if(wantStars) {
+        for(i=0;i<NUMSTARS;i++) {
+            W_DrawPoint(baseWin, stars[i].x, stars[i].y, W_Black);
+        }
     }
     if(drewlevel) {
         W_ClearArea(baseWin, WINWIDTH/2 - (W_StringWidth("LEVEL 000", W_BigFont)/2), WINHEIGHT/2-W_BigTextheight/2,
@@ -154,39 +169,78 @@ static void do_stars()
 {
     int i;
     
-    for(i=0;i<NUMSTARS;i++) {
-        if(!paused)
-            stars[i].y+=stars[i].speed*((starspeed < 20) ? ABS(starspeed) : 20);
-        if(stars[i].y >= WINHEIGHT) {
-            stars[i].y-=WINHEIGHT+starspeed;
-            stars[i].x = random() % WINWIDTH;
-            switch(random()%5) {
-            case 0:
-                stars[i].color = W_White;
-                break;
-            case 1:
-                stars[i].color = W_Green;
-                break;
-            case 2:
-                stars[i].color = W_Cyan;
-                break;
-            case 3:
-                stars[i].color = W_Red;
-                break;
-            default:
-                stars[i].color = W_Yellow;
-                break;
+    if(wantStars) {
+        for(i=0;i<NUMSTARS;i++) {
+            if(!paused)
+                stars[i].y+=stars[i].speed*((starspeed < 20) ? ABS(starspeed) : 20);
+            if(stars[i].y >= WINHEIGHT) {
+                stars[i].y-=WINHEIGHT+starspeed;
+                stars[i].x = random() % WINWIDTH;
+                switch(random()%5) {
+                case 0:
+                    stars[i].color = W_White;
+                    break;
+                case 1:
+                    stars[i].color = W_Green;
+                    break;
+                case 2:
+                    stars[i].color = W_Cyan;
+                    break;
+                case 3:
+                    stars[i].color = W_Red;
+                    break;
+                default:
+                    stars[i].color = W_Yellow;
+                    break;
+                }
             }
+            W_DrawPoint(baseWin, stars[i].x, stars[i].y, stars[i].color);
         }
-        W_DrawPoint(baseWin, stars[i].x, stars[i].y, stars[i].color);
+#ifdef SHOW_SHIELD_BAR
+	    if ((plshield > 0) || (shieldsleft > 0)) { 
+		int shieldcount = 0;
+		int total_shields = (plshield + shieldsleft) * 19 / SHIELDTIME + 1;
+		while (total_shields > 0) {
+			shieldcount++;
+			total_shields -= 19;
+			W_DrawImage(baseWin, WINWIDTH - 20 * shieldcount, 0, 0, shieldImage, W_Cyan);
+		}
+		while (total_shields < 0) {
+			int column;
+			column = WINWIDTH - 20 * shieldcount - total_shields++;
+			W_MakeLine(baseWin, column, 0, column, 20, W_Black);
+			W_MakeLine(baseWin, column - 1, 0, column - 1, 20, W_Black);
+		}
+       	    }
+#endif /* SHOW_SHIELD_BAR */
+
+
     }
     if(starspeed != 1) {
         char buf[20];
+#ifndef ORIGINAL_XGALAGA
+	int y;
+#endif
     
         drewlevel = 1;
         sprintf(buf, "LEVEL %d", level+1);
-        W_MaskText(baseWin, WINWIDTH/2 - (W_StringWidth(buf, W_BigFont)/2), WINHEIGHT/2-W_BigTextheight/2,
-                   W_Red, buf, strlen(buf), W_BigFont);
+#ifndef ORIGINAL_XGALAGA
+	y = WINHEIGHT/2-W_BigTextheight/2;
+#endif
+        W_MaskText(baseWin, WINWIDTH/2 - (W_StringWidth(buf, W_BigFont)/2), WINHEIGHT/2-W_BigTextheight/2, W_Red, buf, strlen(buf), W_BigFont);
+#ifndef ORIGINAL_XGALAGA
+	if (shots > 0) {
+		int x = WINWIDTH/2 - 14 * W_Textwidth;
+		y += W_BigTextheight + 20;
+	
+		sprintf(buf, "Torps: %d  Hits: %d", shots, hits);
+		W_MaskText(baseWin, x, y, W_Yellow, buf, strlen(buf), W_RegularFont);
+	
+		x += 23 * W_Textwidth;
+		sprintf(buf, "(%d%%)", 100 * hits / shots);
+		W_MaskText(baseWin, x, y, W_Green, buf, strlen(buf), W_RegularFont);
+	}
+#endif
     }
 }
 
@@ -424,8 +478,8 @@ static void do_aliens()
         if(convoyx <= 0) {
             convoyx=0;
             convoymove = -convoymove;
-        } else if(convoyx >= WINWIDTH/2) {
-            convoyx = WINWIDTH/2;
+        } else if(convoyx >= (WINWIDTH-180)) {
+            convoyx = WINWIDTH - 180;
             convoymove = -convoymove;
         }
     }
@@ -576,7 +630,7 @@ static void do_aliens()
             }
         }
     }
-    if(livecount == 0) {
+    if(livecount == 0 && !paused) {
         starspeed++;
 #ifdef SOUND
         if(starspeed == 2)
@@ -769,10 +823,13 @@ static void undo_player()
                      playerShip->width, h);
 }
 
-static void do_player()
+static void do_player(mx, my, but)
+int mx, my, but;
 {
-    int mx, my, but;
     static int torpok;
+#ifdef ENABLE_MACHINE_GUN
+    static int shotside = 0;
+#endif
     W_Event wev;
     static int keys = 0;
 
@@ -790,12 +847,38 @@ static void do_player()
                 switch(wev.key) {
                 case 'q':
                 case 'Q':
+                case 256+'Q':   //369:
+                case 256+'q':   //337:
                     xgal_exit(0);
                     break;
                 case 'm':
+                case 256+'m':    //365:
                     mouseControl = 2;
                     break;
+                 case 'k':
+                 case 256+'k':   //363:
+                     mouseControl = 0;
+                     W_UngrabPointer();
+                     break;
+                case '\r'+256:
+                    W_ToggleFullscreen(shellWin);
+                    fullscreen = !fullscreen;
+                    if (fullscreen)
+                        W_BlankCursor(baseWin);
+                    else
+                        W_RevertCursor(baseWin);
+                    return;
+                    break;
+#ifdef SOUND
+                case 's':       /* toggle sound on the title screen */
+                case 256+'s':
+                    playSounds = !playSounds;
+                    return;     /* this key must not start the game */
+                    break;
+#endif
 		default:
+/*printf ("1keyevent %d\n", wev.key); */
+                    return;     /* unhandled key must not cause any action */
 		    break;
                 }
                 if(mouseControl < 2)
@@ -809,6 +892,12 @@ static void do_player()
                     weapon = 0;
                     movespeed = MINSPEED;
                     ships=2;
+#ifdef ACTIVATED_SHIELD
+		    shieldsleft = STARTSHIELDS;
+		    shieldon = 0;
+#else
+		    shieldsleft = 0;
+#endif
                     level=startLevel;  /* change made here */
                     init_aliens(level);
                     gotlemon = 0;
@@ -819,6 +908,13 @@ static void do_player()
                     W_ClearWindow(baseWin);
                 }
                 break;
+            case W_EV_EXPOSE:
+	    	if (wev.Window == shellWin)
+		    draw_score();
+		break;
+            default:
+/*printf ("2keyevent %d\n", wev.key);*/
+              break;
             }
         }
         return;
@@ -837,8 +933,17 @@ static void do_player()
                 keys &= ~(LEFTKEY);
                 break;
             case ' ':
+            case 256+' ':       //288:
                 keys &= ~(FIREKEY);
                 break;
+#ifdef ACTIVATED_SHIELD
+	    case 'x':
+            case 256+'x':
+		shieldsleft += plshield;
+		plshield = 0;
+		shieldon = 0;
+		break;
+#endif
             }
             break;
         case W_EV_KEY:
@@ -850,31 +955,46 @@ static void do_player()
                 keys |= LEFTKEY;
                 break;
             case ' ':
+            case 256+' ':       //288:
                 keys |= FIREKEY;
                 break;
             case 'k':
+            case 256+'k':       //363:
                 mouseControl = 0;
                 W_UngrabPointer();
                 break;
             case 'm':
+            case 256+'m':       //365:
                 mouseControl = 1;
                 W_GrabPointer(baseWin);
                 break;
+#ifdef ACTIVATED_SHIELD
+	    case 'x':
+            case 256+'x':
+		plshield += shieldsleft;
+		shieldsleft = 0;
+		shieldon = 1;
+		break;
+#endif
             case 'q':
-                if(!pldead) {
+            case 256+'q':       //369:
+                if(!pldead  && !paused) {
                     new_explosion(plx, WINHEIGHT - ((playerShip->height)/2), 2);
                     ships = 0;
                     pldead = 1;
                 }
                 break;
             case 'Q':
+            case 256+'Q':       //337:
                 xgal_exit(0);
                 break;
             case 'b':
+            case 256+'b':       //354:
                 W_Buffer(baseWin, !W_IsBuffered(baseWin));
                 W_ClearWindow(baseWin);
                 break;
             case 'p':
+            case 256+'p':       //368:
                 paused=!paused;
                 if(!paused) {
                     undo_pause();
@@ -885,17 +1005,20 @@ static void do_player()
                 break;
 #ifdef SOUND
             case 's':
+            case 256+'s':       //371:
                 playSounds = !playSounds;
                 break;
 #endif
 #ifdef IM_A_BIG_FAT_CHEATER
             case 'i':
+            case 256+'i':       //361:
                 if(plflash >= 0)
                     plflash = -2;
                 else
                     plflash = 0;
                 break;
             case 'l':
+            case 256+'l':       //364:
                 {
                     int i;
                     for(i=0;i<MAXALIENS;i++)
@@ -905,39 +1028,53 @@ static void do_player()
                 }
                 break;
             case 'c':
+            case 256+'c':       //355:
                 score+= BONUSSHIPSCORE;
                 break;
             case 'h':
+            case 256+'h':       //360:
                 plshield = SHIELDTIME;
 #ifdef SOUND
                 play_sound(SND_SHIELD);
 #endif
                 break;
             case 'w':
+            case 256+'w':       //375:
                 weapon++;
-                if(weapon == 3)
+                if(weapon == NUMWEAPONS)
                     weapon=0;
                 break;
             case 't':
+            case 256+'t':       //372:
                 maxtorps++;
                 if(maxtorps > MAXTORPS)
                     maxtorps = MINTORPS;
                 break;
 #endif /* IM_A_BIG_FAT_CHEATER */
+            case '\r'+256:
+                W_ToggleFullscreen(shellWin);
+                fullscreen = !fullscreen;
+                if (fullscreen)
+                    W_BlankCursor(baseWin);
+                else
+                    W_RevertCursor(baseWin);
+                break;
             default:
+/*printf ("3keyevent %d\n", wev.key);*/
                 break;
             }
+        case W_EV_EXPOSE:
+	    if (wev.Window == shellWin)
+	    	draw_score();
+	    break;
         }
     }
 
     if(!paused) {
         torpok--;
 
-	W_GetMouse(baseWin, &mx, &my, &but);
-        if(!mouseControl) {
-	    /*            W_GetMouse(baseWin, &mx, &my, &but);
-        } else {
-	*/
+        if((!mouseControl && ! js_device) ||
+	   (js_device && mx == plx && but == 0)) {
             if(keys & LEFTKEY)
                 mx = 0;
             else if(keys & RIGHTKEY)
@@ -966,10 +1103,51 @@ static void do_player()
 #endif
                     }
                 } else {
+#ifdef DISABLE_RESET_ON_DEATH
+                    ships--;
+                    maxtorps--;
+                    if (maxtorps < MINTORPS)
+                    	maxtorps = MINTORPS;
+                    switch (weapon)
+                    {
+                    	case SINGLESHOT:
+                    		if (maxtorps < 3)
+                    		{
+                    			maxtorps = 3;
+                    			weapon = SINGLESHOT;
+                    		}
+                    		break;
+                    	case DOUBLESHOT:
+	                    	if (maxtorps < 4)
+                    		{
+                    			maxtorps = 4;
+                    		}
+        					break;
+                   		case SPREADSHOT:
+                    		if (maxtorps < 5)
+                    		{
+                    			maxtorps = 5;
+                    		}
+                    		break;
+						case TRIPLESHOT:
+							if (maxtorps < 6)
+                    		{
+                    			maxtorps = 6;
+                    		}
+                    		break;
+                    	case MACHINEGUN:
+                    		if (maxtorps < 3)
+                    		{
+                    			maxtorps = 3;
+                    		}
+                    		break;
+		}
+#else
                     ships--;
                     maxtorps = MINTORPS;
                     weapon = 0;
                     movespeed = MINSPEED;
+#endif /* DISABLE_RESET_ON_DEATH */
                     pldead = 0;
                     plflash = 50;
                     plx = WINWIDTH/2;
@@ -983,11 +1161,13 @@ static void do_player()
             case SINGLESHOT:
                 if(numtorps < maxtorps)
                     new_torp(plx, WINHEIGHT - playerShip->height, 0, -TORPSPEED);
+		    torpok = TORPDELAY;
                 break;
             case DOUBLESHOT:
                 if(numtorps < maxtorps-1) {
                     new_torp(plx-5, WINHEIGHT - playerShip->height, 0, -TORPSPEED);
                     new_torp(plx+5, WINHEIGHT - playerShip->height, 0, -TORPSPEED);
+		    torpok = TORPDELAY;
                 }
                 break;
             case TRIPLESHOT:
@@ -995,11 +1175,62 @@ static void do_player()
                     new_torp(plx-5, WINHEIGHT - playerShip->height, -2, 1-TORPSPEED);
                     new_torp(plx,   WINHEIGHT - playerShip->height, 0,   -TORPSPEED);
                     new_torp(plx+5, WINHEIGHT - playerShip->height, 2, 1-TORPSPEED);
+		    torpok = TORPDELAY;
                 }
                 break;
-            }
-            torpok = TORPDELAY;
-        }
+#ifdef ENABLE_SPREAD_SHOT
+            case SPREADSHOT:
+            	if (numtorps == 0)
+                {
+			if ((maxtorps % 2) == 1)
+				new_torp(plx, WINHEIGHT - playerShip->height, 0, -TORPSPEED*1.15);
+			else
+			{
+				new_torp(plx - 5, WINHEIGHT - playerShip->height, 0, -TORPSPEED*1.15);
+				new_torp(plx + 5, WINHEIGHT - playerShip->height, 0, -TORPSPEED*1.15);
+			}
+			if (maxtorps > 2)
+			{
+				new_torp(plx, WINHEIGHT - playerShip->height - 15, -2, -TORPSPEED*1.15);
+				new_torp(plx, WINHEIGHT - playerShip->height - 15, 2, -TORPSPEED*1.15);
+			}
+			if (maxtorps > 4)
+			{
+				new_torp(plx, WINHEIGHT - playerShip->height - 25, -4, -TORPSPEED*1.15);
+				new_torp(plx, WINHEIGHT - playerShip->height - 25, 4, -TORPSPEED*1.15);
+			}
+			if (maxtorps > 6)
+			{
+				new_torp(plx, WINHEIGHT - playerShip->height - 35, -6, -TORPSPEED*1.15);
+				new_torp(plx, WINHEIGHT - playerShip->height - 35, 6, -TORPSPEED*1.15);
+			}
+			if (maxtorps > 8)
+			{
+				new_torp(plx, WINHEIGHT - playerShip->height - 50, -8, -TORPSPEED*1.15);    
+				new_torp(plx, WINHEIGHT - playerShip->height - 50, 8, -TORPSPEED*1.15);
+			}
+			if (maxtorps > 10)
+			{
+				new_torp(plx, WINHEIGHT - playerShip->height - 60, -10, -TORPSPEED*1.15);
+				new_torp(plx, WINHEIGHT - playerShip->height - 60, 10, -TORPSPEED*1.15);
+                        }
+                        torpok = TORPDELAY;
+                 }
+		 break;
+#endif /* ENABLE_SPREAD_SHOT */
+#ifdef ENABLE_MACHINE_GUN
+            case MACHINEGUN:
+                 if(numtorps < maxtorps)
+                 {
+			shotside = (shotside == -15) ? 15 : -15;
+			new_torp(plx + shotside, WINHEIGHT - playerShip->height, 0, -TORPSPEED * 1.3);
+			torpok = TORPDELAY - 2;
+		}
+		break;
+#endif /* ENABLE_MACHINE_GUN */
+	    }
+	}
+
 
         if(!but)
             torpok=0;
@@ -1008,11 +1239,20 @@ static void do_player()
             plx+=movespeed;
         else if((mx/movespeed) < (plx/movespeed))
             plx-=movespeed;
+#ifdef ENABLE_SHIP_WRAP
+
+        if(plx < 10)
+        	plx=WINWIDTH - 10;
+        if(plx > WINWIDTH - 10)
+			plx=10;
+#else
+  
     
         if(plx < playerShip->width/2)
             plx=playerShip->width/2;
         if(plx> WINWIDTH - playerShip->width/2)
             plx=WINWIDTH - playerShip->width/2;
+#endif
     
         if(plflash > 0)
             plflash--;
@@ -1024,7 +1264,7 @@ static void do_player()
             W_DrawImage(baseWin, plx-(shieldImage->width/2), WINHEIGHT - shieldImage->height - 3, 0, shieldImage,
                         W_Cyan);
         }
-    } else { /* paused */
+    } else if (!pldead) { /* paused */
         W_DrawImage(baseWin, plx-(playerShip->width/2), WINHEIGHT - playerShip->height, counter, playerShip, W_Red);
     }
 }
@@ -1036,12 +1276,12 @@ char  **argv;
 {
     int ac;
     char *dpyname = 0;
-    struct timeval tv, otv;
-    long ut;
+    int mx, my, but;
+    int start_fullscreen = 1;
    
-    gettimeofday(&otv, 0);
     for(ac = 1; ac < argc; ac++) {
         if(*argv[ac] == '-') {
+            int w, h;
             if(strcmp(argv[ac], "-scores") == 0) {
                 print_scores();
                 exit(0);
@@ -1049,10 +1289,8 @@ char  **argv;
             if(strcmp(argv[ac], "-display") == 0 && (ac+1 < argc)) {
                 dpyname = argv[ac+1];
                 ac++;
-#if 0
             } else if (strcmp(argv[ac], "-b") == 0) {
                 useBuffered = !useBuffered;
-#endif
             } else if (strcmp(argv[ac], "-keyboard") == 0) {
                 mouseControl = 0;
             } else if (strcmp(argv[ac], "-mouse") == 0) {
@@ -1063,9 +1301,25 @@ char  **argv;
 #endif
 	/* '-level' option defined here */
             } else if (strcmp(argv[ac], "-level") == 0 && (ac+1 < argc) 
-                       && atoi(argv[ac+1]) >= 1) {
-                startLevel = atoi(argv[ac+1]);
+                       && atoi(argv[ac+1]) >= 1) 
+            {
+                int nlev;
+                nlev = atoi(argv[ac+1]); 
+                if (nlev > 15 ) nlev = 15;
+                startLevel = nlev;
                 ac++;
+            } else if (strcmp(argv[ac], "-nostars") == 0) {
+                wantStars = 0;
+            } else if (strcmp(argv[ac], "-nouseor") == 0) {
+                useOR = 0;
+            } else if (strcmp(argv[ac], "-noclipmask") == 0) {
+                useClipMask = 0;
+            } else if (strcmp(argv[ac], "-window") == 0) {
+                start_fullscreen = 0;
+            } else if ((strcmp(argv[ac], "-winsize") == 0) && (++ac < argc) &&
+                       (sscanf(argv[ac], "%dx%d", &w, &h) == 2)) {
+                WINWIDTH  = w;
+                WINHEIGHT = h;
             } else {
                 print_usage();
                 exit(0);
@@ -1079,8 +1333,18 @@ char  **argv;
     W_Initialize(dpyname);
 
     backColor = W_Black;
-    baseWin = W_MakeWindow("XGalaga", 0, 0, WINWIDTH, WINHEIGHT, 0, "tiny", 0, W_White);
+    WINHEIGHT -= (W_Textheight+1);
+    shellWin = W_MakeWindow("XGalaga", 0, 0, WINWIDTH, WINHEIGHT + W_Textheight+1, 0, "tiny", 0, W_White);
+    baseWin = W_MakeWindow("", 0, W_Textheight+1, WINWIDTH, WINHEIGHT, shellWin, "tiny", 0, W_White);
+    W_Buffer(shellWin, 0);
+    W_MapWindow(shellWin);
     W_MapWindow(baseWin);
+    if (start_fullscreen)
+    {
+      W_ToggleFullscreen(shellWin);
+      W_BlankCursor(baseWin);
+      fullscreen = 1;
+    }
 
     /*    W_AutoRepeatOff();*/
 
@@ -1103,62 +1367,67 @@ char  **argv;
     init_score();
     init_prizes();
 #ifdef SOUND
-    init_sound();
+    init_sound();       /* starts the soundserver */
 #endif
     init_aliens(level);
+    init_framerate();
+#ifdef __linux__
+    init_joystick();
+#endif
 
     ships = 2;
     nextBonus = 20000;
 
     while(1) {
         counter++;
-        if(gameOver && getting_name)
-            undo_name();
 
-        undo_score();
+	/* For the benefit of unbuffered mode, the most important things are
+	 * erased/redrawn closest together so they spend the least time blanked.
+	 * player, aliens and etorps are most important for game play.
+	 * pause, title and name are important in their modes and aren't done
+	 * otherwise.
+	 *
+	 * The title, name, pause and score "extra ship" want to overlay
+	 * everything else drawn, so they come last.
+	 */
+	undo_stars();
         undo_explosions();
-        undo_stars();
-        undo_aliens();
         undo_prizes();
-        undo_etorps();
         undo_torps();
+        undo_etorps();
+        undo_aliens();
         undo_player();
-
-        if(paused)
-            undo_pause();
+        if(gameOver && getting_name) undo_name();
+        if(paused) undo_pause();
+        undo_score();
 
         W_FlushClearAreaCache(baseWin);
 
-        if(paused)
-            do_pause();
-
-        do_stars();
-        do_player();
-        do_torps();
         do_etorps();
-        do_prizes();
+        do_player(mx, my, but);
         do_aliens();
+        do_torps();
+        do_prizes();
         do_explosions();
+ 	do_stars();
         do_score();
-        if(gameOver) {
-            do_title();
-            if(getting_name)
-                do_name();
-        }
+        if(gameOver) { do_title(); if(getting_name) do_name(); }
+        if(paused) do_pause();
+
         W_DisplayBuffer(baseWin);
-        W_Flush();
-        gettimeofday(&tv, 0);
-        if(tv.tv_sec > otv.tv_sec)
-            tv.tv_usec += (tv.tv_sec - otv.tv_sec) * 1000000;
-        ut = tv.tv_usec - otv.tv_usec;
-        if(UTIMER - ut > 0) {
-            struct timeval tv;
-            unsigned long usecs = UTIMER - ut;
-            tv.tv_sec  = usecs / 1000000L;
-            tv.tv_usec = usecs % 1000000L;
-            (void) select (0, 0, 0, 0, &tv);
-	}
+
+	/* This is an XSync style round trip to the server with the bonus of
+	 * getting the mouse position.
+	 * If the server can't draw at the UTIMER frame rate then this will be
+	 * the only delay in the loop.
+	 */
+	W_GetMouse(baseWin, &mx, &my, &but);
+#ifdef __linux__
+	do_joystick(&mx, &my, &but);
+#endif
+        do_framerate();
 
-        gettimeofday(&otv, 0);
     }
+    return (0);
 }
+
