@@ -159,6 +159,8 @@ int paths[NUMPATHS][MAXPATH][2] = {
     { 6,80, -1,-1, },
 };	
 
+/* These aren't used anymore! (see filepath.c) Maybe we should nuke them? */
+#if 0
 /* enter_paths are for new ships.  consists of starting co-ordinates, number of frames
    to wait at the start (presumably offscreen) and a path to follow.
    path -1 means this ship is not used on this level
@@ -1159,6 +1161,8 @@ struct enter_path ep[NUMLEVELS][MAXALIENS] = {
     },
 };
 
+#endif /* This isn't used anymore! Maybe we should nuke it */
+
 void path_dir(int path, int pos, int *dir, int *steer)
 {
     *dir =   paths[path][pos][0];
@@ -1197,8 +1201,6 @@ void start_path(int path, struct alien *al)
 
 void new_alien(int level, int anum, struct alien *al)
 {
-    int l = (level-1) % NUMLEVELS;
-
     if(get_shape(anum) >= 0) {
 	al->alive = 1;
 	al->dying = 0;
